
class PythonModule:
  '''
  A Python module implemented in C
  '''
  
  def __init__(self, obj = None, subClasses = None):
    '''
    A Python module implemented in C
    '''
    self.classes = []
    if obj is None:
      if not subClasses is None:
        self.classes.extend(subClasses)
      import ASE.Compiler.Cxx.Source
      self.parent = ASE.Compiler.Cxx.Source.Source(subClasses = self.classes)
      import ASE.Compiler.Cxx.PythonModule_impl
      self.obj = ASE.Compiler.Cxx.PythonModule_impl.PythonModule(self)
      self.classes.append(self.obj)
      self.parent.classes.insert(-1, self.obj)
    else:
      self.obj = obj
      self.parent = obj
  
  def __getattr__(self, attr):
    '''
    The virtual dispatch is encoded here
    '''
    for klass in self.classes:
      if hasattr(klass, attr):
        return getattr(klass, attr)
    if not self.parent is None:
      return getattr(self.parent, attr)
    raise AttributeError, attr
  
  def getBaseObject(obj):
    '''
    Get the underlying object which might have been cast several times
    '''
    while hasattr(obj, 'obj'):
      obj = obj.obj
    return obj
  getBaseObject = staticmethod(getBaseObject)
  
  def __eq__(self, obj):
    if not hasattr(obj, 'obj'):
      return 0
    return self.isSame(obj)
  
  def __hash__(self):
    '''
    Hashes IOR pointer
    '''
    return hash(id(self.getBaseObject(self)))
  
  def __nonzero__(self):
    '''
    Always return true
    '''
    return 1
  
  def __repr__(self):
    return '<class ' + self.getClassName() + ' at ' + hex(id(self)) + '>'
  
  def __str__(self):
    return '<class ' + self.getClassName() + ' at ' + hex(id(self)) + '>'
  
  def isSame(self, obj):
    '''
    Return true if and only if <code>obj</code> refers to the same underlying object as this object
    '''
    return id(self.getBaseObject(self)) == id(self.getBaseObject(obj))
  
  def getClassName(self):
    '''
    Return the type name of the underlying class object
    '''
    return 'ASE.Compiler.Cxx.PythonModule'
  
  def isInstanceOf(self, typeName):
    '''
    Return <code>true</code> if this object is an instance of the specified type
    - This routine will return <code>true</code> if and only if a cast to the specified type would succeed
    '''
    if len(self.classes) > 1:
      import ASE.Loader
      return ASE.Loader.Loader.createClass(self.classes[0].__module__[:-5]).isInstanceOf(typeName)
    elif len(self.classes) == 0 and not self.obj is None:
      return self.obj.isInstanceOf(typeName)
    elif typeName in ('ASE.BaseInterface', 'ASE.BaseClass', 'ASE.Compiler.Vertex', 'ASE.Compiler.Cxx.Vertex', 'ASE.Compiler.Cxx.Source', 'ASE.Compiler.Cxx.PythonModule'):
      return 1
    return 0
  
  def weakReference(self):
    return self
